/* wstypes.h */

#ifndef WSTYPES_H
#define WSTYPES_H 1

#include <time.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifndef MAXTEMPS
#define MAXTEMPS           8
#endif

#ifndef MAXHUMS
#define MAXHUMS            8
#endif

#ifndef MAXBAROM
#define MAXBAROM           8
#endif

#ifndef MAXGPC
#define MAXGPC             8
#endif

#ifndef MAXSOL
#define MAXSOL             4
#endif

#ifndef MAXLCD
#define MAXLCD             4
#endif

#ifndef MAXBRANCHES
#define MAXBRANCHES        9
#endif

// family codes of devices
#define TEMP_FAMILY        0x10
#define TEMP_FAMILY_DS1822 0x22
#define TEMP_FAMILY_DS18B20 0x28
#define SBATTERY_FAMILY    0x26
#define SWITCH_FAMILY      0x12
#define COUNT_FAMILY       0x1D
#define DIR_FAMILY         0x01
#define BRANCH_FAMILY      0x1F
#define NULL_FAMILY        0xFF
#define ATOD_FAMILY        0X20
#define PIO_FAMILY         0x29
#define HYGROCHRON_FAMILY  0x41
// switch states
#define SWITCH_AON_BON     0x1F
#define SWITCH_AOFF_BON    0x3F
#define SWITCH_AON_BOFF    0x4F
#define SWITCH_AOFF_BOFF   0x7F

#ifndef WSTYPES_H
  #define WSTYPES_H 1
#endif

#ifndef HAVE_UINT32_T
typedef unsigned long uint32_t ;
#endif

#ifndef HAVE_INT32_T
typedef long int32_t ;
#endif

typedef struct {
  unsigned long count ;         /* Latest count value */
  unsigned long count_old ;     /* Previous count value */
  unsigned long count_delta ;   /* Change in counts */
  unsigned long count_offset ;  /* Value to subtract */
  unsigned long subcount ;      /* Count at latest sub-update */
  unsigned long subcount_old ;  /* Count at previous sub-update */
  unsigned long subcount_delta ;/* Difference between last two sub-update counts */
  float max_sub_delta ;         /* Maximum event size */
  unsigned long max_subcount_delta_scratch ;/* Maximum event delta in counts */
  unsigned long event_count ;   /* Change counter (e.g. lightning counter) */
  float         sub_delta ;     /* sub-integration calibrated delta */
  time_t        time_reset ;    /* Time corresponding to count_offset */
  time_t        time ;          /* Time of last read */
  time_t        time_delta ;    /* Time since last read */
  time_t        time_last_inc ; /* Time since last increment */
  float         gpc ;           /* calib[0] * (count - count_offset) */
  float         delta ;         /* count_delta * calib[0] */
  float         rate ;          /* 3600 * delta / time_delta */
} gpc_struct ;

typedef struct {
  unsigned long anem_start_count /* Anemometer counter reading at start */,
                anem_end_count   /* Anemometer counter reading at end */,
                gust_start_count,
                gust_end_count,
                rain_count       /* Rain gauge counter reading */,
                rain_offset[8]   /* Rain gauge offset values */,
                rain_count_old ;
  long anem_start_time, anem_end_time ;
  long gust_start_time, gust_end_time ;
  float T[MAXTEMPS] ;     /* Celsius */
  float RH[MAXHUMS] ;     /* per cent */
  float barom[MAXBAROM] ; /* mBar */
  float Tb[MAXBAROM] ;    /* Celsius */
  float Trh[MAXHUMS] ;    /* Celsius */
  gpc_struct gpc[MAXGPC] ; /* General Purpose Counters */
  float solar[MAXSOL] ;   /* Solar radiation values */
  int /*NumTemps,*/ Tnum, Hnum, gpcnum, solnum ; /* Index for each device type */
  int Bnum;   /* GNC HACKS barometer */
  float anem_speed, anem_mps, anem_gust, rain ;
  int vane_bearing, vane_mode ;
  unsigned char vane1_sn[12], vane2_sn[12]/*, anem_sn[12], rain_sn[12]
    T_sn[MAXTEMPS][12], switch_sn[12]*/ ;
  char rain_date[128] ;
  float rain_rate ; /* Inches per hour */
  time_t t_rain[8] ; /* Unix time each rain reset */
  float Tmin, Tmax, anem_speed_max ; /* "Daily" stats values */
  float anem_int_gust ; /* Max gust during an update interval */
  float anem_int_gust_scratch ; /* Working value for int gust - copied to anem_int_gust at end */
  time_t t ; /* The time at which these values were recorded */
  int delta_t ; /* The change in t */
  int interval ; /* How long till the next results are due */
  float latitude, longitude ; /* Station location */
} wsstruct ;
#endif
